﻿/************************************************************************************
本代码由明德扬工作室精心设计和制作。

我们希望通过规范、严谨的代码，使同学们接触到纯正的集成电路/FPGA代码。

明德扬工作室长期招募成员，可免费培训，欢迎集成电路/FPGA爱好者加入。

学习交流群：97925396

*************************************************************************************/


/*********www.mdy-edu.com 明德扬科教 注释开始****************
模块功能说明：开发板上有一组矩阵键盘，每当有按键按下时，通过
行扫描法来确定是哪个按键按下的。

接口定义：
clk     :   时钟信号，频率是50MHz
rst_n   :   复位信号，在低电平时有效
key_col :   矩阵键盘列信号，默认上拉高电平
key_row :   矩阵键盘行信号，默认输出低电平
key_vld :   按键有效信号，默认输出高电平
key_num :   按键位置信号，默认输出高电平
**********www.mdy-edu.com 明德扬科教 注释结束****************/

module  key_scan(
                 clk    ,
                 rst_n  ,
                 key_col,
                 key_row,
                 key_num,
                 key_vld   
               );

/*********www.mdy-edu.com 明德扬科教 注释开始****************
参数定义，明德扬规范要求，verilog内的用到的数字，都使用参数表示。
参数信号全部大写
**********www.mdy-edu.com 明德扬科教 注释结束****************/
    parameter      KEY_W  =     4      ;
    parameter      CHK_COL  =   0 	   ;
    parameter      CHK_ROW  =   1 		;
    parameter      DELAY    =   2 		;
    parameter      WAIT_END =   3 		;
    parameter      COL_CNT  =   16		;
    parameter      SHAKE_W  =   20		;
    parameter      TIME_20MS=   1000000;

    //输入信号定义
    input               clk    			;
    input               rst_n  			;
    input  [KEY_W-1:0]  key_col			;

    //输出信号定义
    output              key_vld			;
    output [KEY_W-1:0]  key_num			;
    output [KEY_W-1:0]  key_row			;

    //输出信号reg定义
    reg   [KEY_W-1:0]   key_num			;
    reg   [KEY_W-1:0]   key_row			;
    reg                 key_vld			;


    reg   [KEY_W-1:0]   key_col_ff0 	;
    reg   [KEY_W-1:0]   key_col_ff1 	;
    reg   [DELAY-1:0]   key_col_get 	;
    reg   [KEY_W-1:0]   state_c     	;
    reg   [SHAKE_W-1:0] shake_cnt   	;
    reg   [KEY_W-1:0]   state_n     	;
    reg   [DELAY-1:0]   row_index   	;
    reg   [COL_CNT-1:0] row_cnt     	;
    wire                shake_flag  	;
    reg                 shake_flag_ff0 ;

/*********www.mdy-edu.com 明德扬科教 注释开始****************
异步处理，打两拍
**********www.mdy-edu.com 明德扬科教 注释结束****************/

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_ff0 <= 4'b0000;
        key_col_ff1 <= 4'b0000;
    end
    else begin
        key_col_ff0 <= key_col    ;
        key_col_ff1 <= key_col_ff0;
    end
end

/*********www.mdy-edu.com 明德扬科教 注释开始****************
消抖 20MS
**********www.mdy-edu.com 明德扬科教 注释结束****************/
assign shake_flag = shake_cnt>=TIME_20MS-1;


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        shake_cnt <= 0;
    end
    else if(key_col_ff1!=4'h0)begin
        if(shake_flag)
            shake_cnt <= shake_cnt;
        else
            shake_cnt <= shake_cnt + 1;
    end
    else begin
        shake_cnt <= 0;
    end
end

/*********www.mdy-edu.com 明德扬科教 注释开始****************
状态机设计
CHK_COL：检查列线是否有低电平，并且没有抖动，保持20ms以上
CHK_ROW: 逐个将行线置低电平，检查列线是否有低电平
DELAY：  由于输出行线，再得到列线，有一定延时
WAIT_END：等待结束，即列线全部为1
**********www.mdy-edu.com 明德扬科教 注释结束****************/
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        state_c <= CHK_COL;
    end
    else begin
        state_c <= state_n;
    end
end

always  @(*)begin
    case(state_c)
        CHK_COL: begin
                     if(shake_flag && shake_flag_ff0==1'b0)begin
                         state_n = CHK_ROW;
                     end
                     else begin
                         state_n = CHK_COL;
                     end
                 end
        CHK_ROW: begin
                     if(row_index==3 && row_cnt==0)begin
                         state_n = DELAY;
                     end
                     else begin
                         state_n = CHK_ROW;
                     end
                 end
        DELAY :  begin
                     if(row_cnt==0)begin
                         state_n = WAIT_END;
                     end
                     else begin
                         state_n = DELAY;
                     end
                 end
        WAIT_END: begin
                     if(key_col_ff1==4'h0)begin
                         state_n = CHK_COL;
                     end
                     else begin
                         state_n = WAIT_END;
                     end
                  end
       default: state_n = CHK_COL;
    endcase
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_row <= 4'b1111;
    end
    else if(state_c==CHK_ROW)begin
        key_row <= ~(1'b1 << row_index);//4'b0;
    end
    else begin
        key_row <= 4'b1111;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        row_index <= 0;
    end
    else if(state_c==CHK_ROW)begin
       if(row_cnt==0)begin
           if(row_index==3)
               row_index <= 0;
           else
               row_index <= row_index + 1;
       end
    end
    else begin
        row_index <= 0;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        row_cnt <= COL_CNT;
    end
    else if(state_c==CHK_ROW || state_c==DELAY) begin
        if(row_cnt==0)begin
            row_cnt <= COL_CNT;
        end
        else begin
            row_cnt <= row_cnt - 1;
        end
    end
    else begin
        row_cnt <= COL_CNT;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        shake_flag_ff0 <= 1'b0;
    end
    else begin
        shake_flag_ff0 <= shake_flag;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_get <= 0;
    end
    else if(state_c==CHK_COL && shake_flag==1'b1 && shake_flag_ff0==1'b0) begin
        if(key_col_ff1==4'b0001)
            key_col_get <= 0;
        else if(key_col_ff1==4'b0010)
            key_col_get <= 1;
        else if(key_col_ff1==4'b0100)
            key_col_get <= 2;
        else 
            key_col_get <= 3;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_num <=15;
    end
    else if(state_c==CHK_ROW && row_cnt==0)begin
        key_num <= {row_index,key_col_get};
    end
    else begin
        key_num <= 15;
    end
end
reg key_vld_1 ;
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_vld <= 1'b0;
    end
    else if(state_c==CHK_ROW && row_cnt==0 && key_col_ff1[key_col_get]==1'b0)begin
        key_vld <= 1'b1;
    end
    else begin
        key_vld <= 1'b0;
    end
end
endmodule

