module seg_display(
    clk    ,
    rst_n  ,
    din    ,
    din_vld,
    segment,
    seg_sel
    );/*synthesis noprune*/

    `include "clogb2.v"
    //
    parameter      SEG_W     =        8				;
    parameter      SEG_NUM   =        8				;
    parameter      DIS_NUM   =       10				;
    parameter      TIME_20US =     1000				;
    parameter  NUM_0          =       8'b0000_0011 ;
	 parameter  NUM_1          =       8'b1001_1111 ;
	 parameter  NUM_2          =       8'b0010_0101 ;
	 parameter  NUM_3          =       8'b0000_1101 ;
	 parameter  NUM_4          =       8'b1001_1001 ;
	 parameter  NUM_5          =       8'b0100_1001 ;
	 parameter  NUM_6          =       8'b0100_0001 ;
	 parameter  NUM_7          =       8'b0001_1111 ; 
	 parameter  NUM_8          =       8'b0000_0001 ;
	 parameter  NUM_9          =       8'b0000_1001 ;
    parameter      SEG_NUM_C  = clogb2(SEG_NUM)-1  ;


    //źŶ
    input                 clk    	;
    input                 rst_n  	;
    input[4*SEG_NUM-1:0]  din    	;
    input[SEG_NUM-1:0]    din_vld	;

    //źŶ
    output                segment	;
    output                seg_sel	;
    

    //źreg
    reg   [SEG_W-1:0]     segment	;	
    reg   [SEG_NUM-1:0]   seg_sel	;

    wire  [3:0]           seg_tmp	;
    reg   [4*SEG_NUM-1:0] din_ff0	;
    reg                   flag   	;
    reg   [9:0]           clk_cnt	;
    reg   [2:0]           sel_cnt	;
    integer               i         ;   
    
    wire                clk_cnt_add ;
    wire                clk_cnt_end ;
    wire                sel_cnt_add ;
    wire                sel_cnt_end ;

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            segment <= NUM_0;
        end
        else begin
            case(seg_tmp)
                4'd0 : segment <= NUM_0;
                4'd1 : segment <= NUM_1;
                4'd2 : segment <= NUM_2;
                4'd3 : segment <= NUM_3;
                4'd4 : segment <= NUM_4;
                4'd5 : segment <= NUM_5;
                4'd6 : segment <= NUM_6;
                4'd7 : segment <= NUM_7;
                4'd8 : segment <= NUM_8;
                4'd9 : segment <= NUM_9;
                default : segment <= NUM_0;
            endcase
        end
    end

    assign   seg_tmp = din_ff0[(sel_cnt+1)*4-1 -:4];


    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            din_ff0 <= 0;
        end
        else begin
            for(i=0;i<SEG_NUM;i=i+1)begin
                if(din_vld[i]==1'b1)begin
                    din_ff0[(i+1)*4-1 -:4] <= din[(i+1)*4-1 -:4];
                end
                else begin
                    din_ff0[(i+1)*4-1 -:4] <= din_ff0[(i+1)*4-1 -:4];
                end
            end
        end
    end

    always  @(*)begin
        flag = 1;
    end

    //1ڴ󲿷ּ
    assign clk_cnt_add = flag == 1;       //1
    assign clk_cnt_end = clk_cnt_add && clk_cnt == TIME_20US - 1;       //βֵ

    always @(posedge clk or negedge rst_n)begin
        if (!rst_n)begin
            clk_cnt <= 0;
        end
        else if(clk_cnt_add)begin
            if(clk_cnt_end)
                clk_cnt <= 0;
            else
                clk_cnt <= clk_cnt + 1;
        end
    end


    //1ڴ󲿷ּ
    assign sel_cnt_add = clk_cnt_end;       //1
    assign sel_cnt_end = sel_cnt_add && sel_cnt == SEG_NUM - 1;       //βֵ

    always @(posedge clk or negedge rst_n)begin
        if (!rst_n)begin
            sel_cnt <= 0;
        end
        else if(sel_cnt_add)begin
            if(sel_cnt_end)
                sel_cnt <= 0;
            else
                sel_cnt <= sel_cnt + 1;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            seg_sel <= {SEG_NUM{1'b1}};
        end
        else begin
            seg_sel <= ~(1'b1 << sel_cnt);
        end
    end


    endmodule

